//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Written by Philippe Leefsma - Autodesk Developer Technical Services. 2010.
// 
// Feedback and bug report: Philippe.Leefsma@Autodesk.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

using namespace std;
#include <vector>
#include <string>

class AcDbAssocActionParam;
class AcDbAssocPathActionParam;
class AcDbDependency;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A small utility class for storing Name-Value properties
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class ObjectProperty
{

public:

	wstring name;
	wstring value;

	ObjectProperty(wstring pName = L" ", wstring pValue = L"")
	{
		name = pName;
		value = pValue;
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A class that stores information about a non db-resident entity in order to identify it at a later time.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class ItemReference
{

public:

	enum ItemReferenceType
	{
		kAcDbEdgeRefReference = 0,
		kUndefReference
	};

	ItemReferenceType referenceType;

	ItemReference()
	{}

	virtual ~ItemReference()
	{}

};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Stores information about an AcDbEdgeRef entity
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class EdgeRefReference: public ItemReference
{
public:

	AcDbObjectId paramId;
	int iIndex;
	int jIndex;

	EdgeRefReference(const AcDbObjectId& ParamId, int i, int j)
	{
		referenceType = kAcDbEdgeRefReference;

		paramId = ParamId;
		iIndex = i;
		jIndex = j;
	}

	virtual ~EdgeRefReference()
	{}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A utility class for analyzing surfaces
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class CAssocSurfacesUtil
{

public:

	CAssocSurfacesUtil(void);

	~CAssocSurfacesUtil(void);


	wstring curveTypeToStr(AcGe::EntityId type);

	wstring lineWeightToStr(AcDb::LineWeight lineWeight);

	wstring CAssocSurfacesUtil::variantToStr(const AcDbEvalVariant& eval);

	void getAllSurfaces(const AcDbObjectId& spaceId, vector<AcDbObjectId>& surfaceIds);

	void getProperties(const AcDbObjectId& id, vector<ObjectProperty>& properties);
	
	void getProperties(ItemReference* pItemRef, vector <ObjectProperty>& properties);

	void getSurfaceProperties(AcDbSurface* surface, vector <ObjectProperty>& properties);

	void getAssocActionBodyProperties(AcDbAssocActionBody* actionBody, vector <ObjectProperty>& properties);

	void getAssocPathActionParamProperties(AcDbAssocPathActionParam* assocPathActionParam, vector <ObjectProperty>& properties);

	void getAssocActionParamProperties(AcDbAssocActionParam* assocActionParam, vector <ObjectProperty>& properties);

	void getEdgeRefProperties(EdgeRefReference* edgeRefReference, vector <ObjectProperty>& properties);

	void getDependencyProperties(AcDbAssocDependency* dependency, vector <ObjectProperty>& properties);
};
