#include "StdAfx.h"
#include "AcConstrainedGeometry.h"
#include "ConstrainDbgHelper.h"

CConstrainDbgHelper::CConstrainDbgHelper(void)
{
}

CConstrainDbgHelper::~CConstrainDbgHelper(void)
{
}


LPCTSTR CConstrainDbgHelper::objToClassStr(const AcRxObject* obj)
{
    ASSERT(obj != NULL);

    AcRxClass* rxClass = obj->isA();
    if (rxClass == NULL)
	{
        return _T("*Unknown*");
    }

    return obj->isA()->name();
}


/***********************************************************************
**
**  CConstrainDbgHelper::objToHandleStr
**      Obtain an old-style handle string from an object pointer.
**
**  **jma
**
****************************************/

LPCTSTR CConstrainDbgHelper::objToHandleStr(const AcDbObject* obj, CString& str)
{
    ASSERT(obj != NULL);

    AcDbHandle handle;

    obj->getAcDbHandle(handle);
    TCHAR tmpStr[256];
    handle.getIntoAsciiBuffer(tmpStr);
    str = tmpStr;
    return str;
}

LPCTSTR CConstrainDbgHelper::objToClassAndHandleStr(const AcDbObject* obj, CString& str)
{
    CString tmpStr;

    str.Format(_T("<%-20s %-4s>"),
                CConstrainDbgHelper::objToClassStr(obj), CConstrainDbgHelper::objToHandleStr(obj, tmpStr));

    return str;
}


/***********************************************************************
**
**  CConstrainDbgHelper::objToEnameStr
**      Obtain an old-style ename string from an object pointer.
**
**  **jma
**
****************************************/

LPCTSTR CConstrainDbgHelper::objToEnameStr(const AcDbObject* obj, CString& str)
{
    ASSERT(obj != NULL);

    ads_name ent;
    acdbGetAdsName(ent, obj->objectId());
    str.Format(_T("%lx"), ent[0]);
    return str;
}


/****************************************************************************
**
**  CConstrainDbgHelper::dbToStr
**
**  **jma
**
*************************************/

LPCTSTR CConstrainDbgHelper::dbToStr(AcDbDatabase* db, CString& str)
{
    if (db == NULL) {
        str = _T("NULL");
        return str;
    }

    const TCHAR* fname;
    Acad::ErrorStatus es = db->getFilename(fname);
    if (es == Acad::eOk)
        str.Format(_T("%p  \"%s\""), db, fname);
    else {
            // see if we can get name from a document
        AcApDocument* doc = acDocManager->document(db);
        if (doc) {
            str.Format(_T("%p  \"%s\""), db, doc->fileName());
        }
        else {
                // last resort, just use the pointer value.  eNotApplicable
                // happens frequently on temporary databases, otherwise we
                // would like to know what is going on.
            if (es == Acad::eNotApplicable)
                str.Format(_T("%p"), db);
            else {
                ASSERT(0);
                str.Format(_T("%p  %s"), db, CConstrainDbgHelper::rxErrorStr(es));
            }
        }
    }

    return str;
}

/****************************************************************************
**
**  CConstrainDbgHelper::rxErrorStr
**
**  **jma
**
*************************************/

const TCHAR* CConstrainDbgHelper::rxErrorStr(Acad::ErrorStatus msgId)
{
    return acadErrorStatusText(msgId);
}


/***********************************************************************
**
**  CConstrainDbgHelper::mergeStyleToStr
**
**  **jma
**
****************************************/

LPCTSTR CConstrainDbgHelper::mergeStyleToStr(AcDb::DuplicateRecordCloning type, CString& str)
{
    if (type == AcDb::kDrcNotApplicable)
        str = _T("Not Applicable");
    else if (type == AcDb::kDrcIgnore)
        str = _T("Ignore");
    else if (type == AcDb::kDrcReplace)
        str = _T("Replace");
    else if (type == AcDb::kDrcXrefMangleName)
        str = _T("Xref Mangle Name");
    else if (type == AcDb::kDrcMangleName)
        str = _T("Mangle Name");
    else if (type == AcDb::kDrcUnmangleName)
        str = _T("Unmangle Name");
    else {
        //ASSERT(0);
        str.Format(_T("*Unknown* (%d)"), type);
    }

    return str;
}

/****************************************************************************
**
**  CConstrainDbgHelper::booleanToStr
**
**  **jma
**
*************************************/

LPCTSTR CConstrainDbgHelper::booleanToStr(bool b, CString& str)
{
    if (b)
        str = _T("True");
    else
        str = _T("False");

    return str;
}

/***********************************************************************
**
**  CConstrainDbgHelper::dwgVersionToStr
**
**  **jma
**
****************************************/

LPCTSTR CConstrainDbgHelper::dwgVersionToStr(AcDb::AcDbDwgVersion type, CString& str)
{
    CString verStr;

    if (type == AcDb::kDHL_MC0_0)
        verStr = _T("kDHL_MC0_0");
    else if (type == AcDb::kDHL_AC1_2)
        verStr = _T("kDHL_AC1_2");
    else if (type == AcDb::kDHL_AC1_40)
        verStr = _T("kDHL_AC1_40");
    else if (type == AcDb::kDHL_AC1_50)
        verStr = _T("kDHL_AC1_50");
    else if (type == AcDb::kDHL_AC2_20)
        verStr = _T("kDHL_AC2_20");
    else if (type == AcDb::kDHL_AC2_10)
        verStr = _T("kDHL_AC2_10");
    else if (type == AcDb::kDHL_AC2_21)
        verStr = _T("kDHL_AC2_21");
    else if (type == AcDb::kDHL_AC2_22)
        verStr = _T("kDHL_AC2_22");
    else if (type == AcDb::kDHL_1001)
        verStr = _T("kDHL_1001");
    else if (type == AcDb::kDHL_1002)
        verStr = _T("kDHL_1002  (AutoCAD 2.5)");
    else if (type == AcDb::kDHL_1003)
        verStr = _T("kDHL_1003  (AutoCAD 2.6)");
    else if (type == AcDb::kDHL_1004)
        verStr = _T("kDHL_1004  (Release 9)");
    else if (type == AcDb::kDHL_1005)
        verStr = _T("kDHL_1005");
    else if (type == AcDb::kDHL_1006)
        verStr = _T("kDHL_1006  (Release 10)");
    else if (type == AcDb::kDHL_1007)
        verStr = _T("kDHL_1007");
    else if (type == AcDb::kDHL_1008)
        verStr = _T("kDHL_1008");
    else if (type == AcDb::kDHL_1009)
        verStr = _T("kDHL_1009  (R11 and R12)");
    else if (type == AcDb::kDHL_1010)
        verStr = _T("kDHL_1010");
    else if (type == AcDb::kDHL_1011)
        verStr = _T("kDHL_1011");
    else if (type == AcDb::kDHL_1012)
        verStr = _T("kDHL_1012  (Release 13)");
    else if (type == AcDb::kDHL_1013)
        verStr = _T("kDHL_1013  (R14 Mid Version)");
    else if (type == AcDb::kDHL_1014)
        verStr = _T("kDHL_1014  (R14 Final Version)");
    else if (type == AcDb::kDHL_1500)
        verStr = _T("kDHL_1500  (R2000 Development)");
    else if (type == AcDb::kDHL_1015)
        verStr = _T("kDHL_1015  (R2000 Final Version)");
    else if (type == AcDb::kDHL_1800a)
        verStr = _T("kDHL_1800a (R2004 Development Version)");
     else if (type == AcDb::kDHL_1800)
        verStr = _T("kDHL_1800 (R2004  Final Version)");
     else if (type == AcDb::kDHL_2100a)
        verStr = _T("kDHL_2100a (R2007 Development Version)");
     else if (type == AcDb::kDHL_1021)
        verStr = _T("kDHL_1021 (R2007 Final Version)");
	 else if (type == AcDb::kDHL_1024)
        verStr = _T("kDHL_1024 (R2010 Final Version)");
    else 
	{
        ASSERT(0);
        ASSERT(AcDb::kDHL_Newest == AcDb::kDHL_1024); // update needed above
        verStr = _T("*Unknown*");
    }

    str.Format(_T("%d = %s"), type, verStr);
    return str;
}

/****************************************************************************
**
**  CConstrainDbgHelper::intToStr
**
**  **jma
**
*************************************/

LPCTSTR CConstrainDbgHelper::intToStr(const int intVal, CString& str)
{
    str.Format(_T("%d"), intVal);
    return str;
}

LPCTSTR CConstrainDbgHelper::dwgDataTypeToStr(const DwgDataType type, CString& str)
{
	 CString verStr;

	 if(type == kDwgNull)
	{
		verStr = _T("Null");
	}
	else if(type == kDwgReal)
	{
		verStr = _T("Real");
	}
	else if(type == kDwgInt32)
	{
		verStr = _T("Int32");
	}
	else if(type == kDwgInt16)
	{
		verStr = _T("Int16");
	}
	else if(type == kDwgInt8)
	{
		verStr = _T("Int8");
	}
	else if(type == kDwgText)
	{
		verStr = _T("Text");
	}
	else if(type == kDwgBChunk)
	{
		verStr = _T("Chunk");
	}
	else if(type == kDwgHandle)
	{
		verStr = _T("Handle");
	}
	else if(type == kDwgHardOwnershipId)
	{
		verStr = _T("HardOwnershipId");
	}
	else if(type == kDwgSoftOwnershipId)
	{
		verStr = _T("SoftOwnershipId");
	}
	else if(type == kDwgHardPointerId)
	{
		verStr = _T("HardPointerId");
	}
	else if(type == kDwgSoftPointerId)
	{
		verStr = _T("SoftPointerId");
	}
	else if(type == kDwg3Real)
	{
		verStr = _T("Real");
	}
	else if(type == kDwgInt64)
	{
		verStr = _T("Int 64");
	}
	else if(type == kDwgNotRecognized)
	{
		verStr = _T("Not Recognized");
	}

	str.Format(_T("%s"), verStr);
	 return str;
}

LPCTSTR CConstrainDbgHelper::AcDbEvalVariantToStr(const AcDbEvalVariant value, CString& str)
{
	 CString verStr;
	 DwgDataType type = value.getType();

	 if(type == kDwgNull)
	{
		verStr = _T("");
	}
	else if(type == kDwgReal)
	{	
		verStr.Format(_T("%.4lf"), value.resval.rreal);
	}
	else if(type == kDwgInt32)
	{
		verStr.Format(_T("%d"), value.resval.rint);
	}
	else if(type == kDwgInt16)
	{
		verStr.Format(_T("%d"), value.resval.rint);
	}
	else if(type == kDwgInt8)
	{
		verStr = _T("Int8");
	}
	else if(type == kDwgText)
	{
		verStr.Format(_T("%s"), value.resval.rstring);
	}
	else if(type == kDwgBChunk)
	{
		verStr = _T("Chunk");
	}
	else if(type == kDwgHandle)
	{
		verStr = _T("Handle");
	}
	else if(type == kDwgHardOwnershipId)
	{
		verStr = _T("HardOwnershipId");
	}
	else if(type == kDwgSoftOwnershipId)
	{
		verStr = _T("SoftOwnershipId");
	}
	else if(type == kDwgHardPointerId)
	{
		verStr = _T("HardPointerId");
	}
	else if(type == kDwgSoftPointerId)
	{
		verStr = _T("SoftPointerId");
	}
	else if(type == kDwg3Real)
	{
		verStr = _T("Real");
	}
	else if(type == kDwgInt64)
	{
		verStr = _T("Int 64");
	}
	else if(type == kDwgNotRecognized)
	{
		verStr = _T("Not Recognized");
	}

	str.Format(_T("%s"), verStr);
	return str;
}

/****************************************************************************
**
**  CConstrainDbgHelper::ptToStr (3D)
**
**  **jma
**
*************************************/
LPCTSTR CConstrainDbgHelper::ptToStr(const AcGePoint3d& pt, CString& str, int unit, int prec)
{
    TCHAR xstr[100], ystr[100], zstr[100];

    acdbRToS(pt.x, unit, prec, xstr);
    acdbRToS(pt.y, unit, prec, ystr);
    acdbRToS(pt.z, unit, prec, zstr);

    str.Format(_T("(%s, %s, %s)"), xstr, ystr, zstr);
    return str;
}


LPCTSTR CConstrainDbgHelper::ImplicitPointTypeToStr(const AcConstrainedImplicitPoint::ImplicitPointType& type, CString& str)
{
	 CString verStr;

	if(type == AcConstrainedImplicitPoint::kStartImplicit)
	{
		verStr = _T("Start point");
	}
	else if(type == AcConstrainedImplicitPoint::kEndImplicit)
	{
		verStr = _T("End point");
	}
	else if(type == AcConstrainedImplicitPoint::kMidImplicit)
	{
		verStr = _T("Mid point");
	}
	else if(type == AcConstrainedImplicitPoint::kCenterImplicit)
	{
		verStr = _T("Center point");
	}
	else if(type == AcConstrainedImplicitPoint::kDefineImplicit)
	{
		verStr = _T("Define Implicit point");
	}

	str.Format(_T("%s"), verStr);
	return str;
}


/****************************************************************************
**
**  CConstrainDbgHelper::doubleToStr
**
**  **jma
**
*************************************/
LPCTSTR CConstrainDbgHelper::doubleToStr(const double realVal, CString& str, int unit, int prec)
{
    TCHAR tmpStr[100];

    acdbRToS(realVal, unit, prec, tmpStr);
    str = tmpStr;
    return str;
}

/****************************************************************************
**
**  CConstrainDbgHelper::selectEntity
**
**  **jma
**
*************************************/

AcDbEntity* CConstrainDbgHelper::selectEntity(LPCTSTR prompt, AcDb::OpenMode openMode)
{
    int rc;
    ads_name ename;
    ads_point pt;

    if (prompt)
        rc = acedEntSel(prompt, ename, pt);
    else
        rc = acedEntSel(_T("\nSelect an entity: "), ename, pt);

    if (rc != RTNORM){
        acutPrintf(_T("\nNothing selected."));
        return NULL;
    }

    AcDbObjectId objId;
    Acad::ErrorStatus es = CConstrainDbgHelper::enameToObjId(ename, objId);
    if (es != Acad::eOk)
        return NULL;

    AcDbEntity* ent;
    es = acdbOpenAcDbEntity(ent, objId, openMode);
    if (es != Acad::eOk) 
	{
        acutPrintf(_T("\nARX ERROR: %s"), acadErrorStatusText(es));
        return NULL;
    }
    else
        return ent;
}

/****************************************************************************
**
**  CConstrainDbgHelper::selectEntity
**
**  **jma
**
*************************************/

Acad::ErrorStatus CConstrainDbgHelper::enameToObjId(ads_name ent, AcDbObjectId& objId)
{
    Acad::ErrorStatus es = acdbGetObjectId(objId, ent);
    ASSERT(es == Acad::eOk);
    if (es != Acad::eOk)
         acutPrintf(_T("\nARX ERROR: %s"), acadErrorStatusText(es));
    
    return es;
}
