// ConstrainListDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ConstrainListDlg.h"
#include "ConstrainDbgHelper.h"

// CConstrainListDlg dialog

IMPLEMENT_DYNAMIC(CConstrainListDlg, CAdUiDialog)

CConstrainListDlg::CConstrainListDlg(CWnd* pParent /*=NULL*/)
	: CAdUiDialog(CConstrainListDlg::IDD, pParent)
{
	m_currentSpaceId = AcDbObjectId::kNull;
	m_obj = NULL;
}

void CConstrainListDlg::SetObj(AcDbObject* obj)
{
	m_obj = obj;
    //{{AFX_DATA_INIT(ArxDbgUiDlgObjState)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
	 

}


CConstrainListDlg::~CConstrainListDlg()
{
	for(int nIndex = 0; nIndex < m_treeObjectArray.length(); nIndex++)
	{
		CConstrainTreeItem *pItem = m_treeObjectArray.at(nIndex);
		delete pItem;
	}

	m_treeObjectArray.removeAll();
}

void CConstrainListDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CONSTRAIN_TREE, m_ConstrainTree);
	DDX_Control(pDX, IDC_CONSTRAIN_LIST, m_ConstrainList);
	DDX_Control(pDX, IDC_COMBO_SPACE, m_SpaceComboBox);
}


BEGIN_MESSAGE_MAP(CConstrainListDlg, CDialog)
	ON_NOTIFY(TVN_SELCHANGED, IDC_CONSTRAIN_TREE, &CConstrainListDlg::OnTvnSelchangedConstrainTree)
	ON_CBN_SELCHANGE(IDC_COMBO_SPACE, &CConstrainListDlg::OnCbnSelchangeComboSpace)
END_MESSAGE_MAP()


// CConstrainListDlg message handlers

BOOL CConstrainListDlg::OnInitDialog()
{
	CAdUiDialog::OnInitDialog();

	// TODO:  Add extra initialization here
	//List the constrain present in the database.
	AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
	{
		//load all the block table 
		AcDbObjectPointer<AcDbBlockTable> blockTbl(pDatabase->blockTableId(), kForRead);

		AcDbBlockTableIterator* pIterator = NULL;
		blockTbl->newIterator(pIterator);
		AcDbObjectId Id;
		const TCHAR* symName;
		while(pIterator->done() == false)
		{
			pIterator->getRecordId(Id);

			AcDbObjectPointer<AcDbBlockTableRecord> bTableRecord(Id, kForRead);
			bTableRecord->getName(symName);

			//add the name
			m_SpaceComboBox.AddString(symName);
			pIterator->step();
		}

		delete pIterator;

		m_SpaceComboBox.SetCurSel(0);
	}

	 buildColumns(m_ConstrainList);
	OnCbnSelchangeComboSpace();
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}



HTREEITEM CConstrainListDlg::addOneTreeItem(LPCTSTR name, const AcDbObjectId& objId, HTREEITEM parent, bool sort, const bool &bConstraintGroupNode, const AcConstraintGroupNodeId &nodeId)
{
	
	CConstrainTreeItem *pItem = new CConstrainTreeItem();
	pItem->m_bConstraintGroupNode = bConstraintGroupNode;
	pItem->m_Id = objId;
	
	if(bConstraintGroupNode == true)
	{
		pItem->m_GroupNodeId = nodeId;
	}
	

	m_treeObjectArray.append(pItem);

	int index = m_treeObjectArray.length() - 1;
	TV_ITEM    tvItem;
    TV_INSERTSTRUCT tvInsert;

    tvItem.mask = TVIF_TEXT | TVIF_PARAM;
    tvItem.pszText = const_cast<TCHAR*>(name);
    tvItem.cchTextMax = lstrlen(name);
    tvItem.lParam = (LPARAM)index;        //index of AcDbObjectId
    
    tvInsert.item = tvItem;
    if (sort)
        tvInsert.hInsertAfter = TVI_SORT;
    else
        tvInsert.hInsertAfter = TVI_LAST;

    tvInsert.hParent = parent;

    return m_ConstrainTree.InsertItem(&tvInsert);   
}


AcDbObjectId CConstrainListDlg::getAssocNetwork(bool createIfNotExist, const AcDbObjectId &spaceId, AcDbDatabase* pDatabase)
{
	Acad::ErrorStatus eStatus = eOk;

	if (pDatabase == NULL)
	{
		pDatabase = acdbHostApplicationServices()->workingDatabase();
	    if (!pDatabase)
		    return AcDbObjectId::kNull;
	}

	//get the existing network
	return AcDbAssocNetwork::getInstanceFromObject(spaceId, createIfNotExist);	
}

void CConstrainListDlg::AddAssocAction(const AcDbObjectId &networkId, HTREEITEM parent)
{
	AcDbObjectPointer<AcDbAssocNetwork> pNetwork(networkId, kForRead);

	if (pNetwork.openStatus() != eOk)
			return ;

	const AcDbObjectIdArray& actionsInNetwork = pNetwork->getActions();
	for (int nCount = 0; nCount < actionsInNetwork.length(); nCount++)
	{
		const AcDbObjectId& actionId = actionsInNetwork[nCount];

		if (actionId == AcDbObjectId::kNull)
				continue;

		AcDbObjectPointer<AcDbAssocAction> pAction(actionId, kForRead);
			
		if (pAction.openStatus() != eOk)
			 continue;

		AcDbAssocAction *pAcation = pAction.object();

		CString str;
		HTREEITEM ConstraintGroupItem = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pAcation, str), actionId, parent, false);

		if(pAcation->isKindOf(AcDbAssoc2dConstraintGroup::desc()) == true)
		{
			
			AcDbAssoc2dConstraintGroup*  pConsGrp = 
				(AcDbAssoc2dConstraintGroup*)(pAction.object());
			
			if (!pConsGrp)
				continue;

			AddGeomConstraint(pConsGrp, ConstraintGroupItem);

		}
		else if(pAcation->isKindOf(AcDbAssocNetwork::desc()) == true)
		{
			//No action required
		}
		else if(pAcation->isKindOf(AcDbAssocVariable::desc()) == true)
		{
			//No action required
		}
			
	}
}
void CConstrainListDlg::OnTvnSelchangedConstrainTree(NMHDR *pNMHDR, LRESULT *pResult)
{
	//LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	// TODO: Add your control notification handler code here
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
    displayCurrent(static_cast<int>(pNMTreeView->itemNew.lParam));  

	*pResult = 0;
}

void CConstrainListDlg::displayCurrent(int index)
{
		// remove any previous entries
    m_fieldStrList.RemoveAll();
    m_valueStrList.RemoveAll();

	AcDbObjectId selectedId = AcDbObjectId::kNull;

    ASSERT((index >= 0) && (index < m_treeObjIdList.length()));
    //selectedId = m_treeObjIdList[index];
	CConstrainTreeItem *pItem = m_treeObjectArray[index];

	if(pItem->m_bConstraintGroupNode == false)
	{
		AcDbObjectPointer<AcDbObject> pObject(pItem->m_Id, kForRead);
		
		//list the 
		if(pObject.openStatus() == Acad::eOk)
		{
			display(pObject.object());
			drawPropsList(m_ConstrainList);
		}
		
	}
	else
	{
		//get the group object id and find the node id....
		AcDbObjectPointer<AcDbAssoc2dConstraintGroup> pGroup(pItem->m_Id, kForRead);
		AcConstraintGroupNode *pNode = pGroup->getGroupNodePtr(pItem->m_GroupNodeId);
		
		display_AcConstraintGroupNode(pNode);
		drawPropsList(m_ConstrainList);
		

	}
   
}

void CConstrainListDlg::addGeometryDependencyObjects(AcConstrainedGeometry *pGeometry, AcDbAssoc2dConstraintGroup *pGroup, HTREEITEM parent)
{
	AcDbObjectId dependencyObjectId = AcDbObjectId::kNull;
	CString str = _T("");
	if(pGeometry->isKindOf(AcConstrainedPoint::desc()) == true)
	{
		if(pGeometry->isKindOf(AcConstrainedImplicitPoint::desc()) == true)
		{
			AcConstrainedImplicitPoint *pImplicitPoint = (AcConstrainedImplicitPoint*)pGeometry;

			AcConstraintGroupNodeId nodeId = pImplicitPoint->constrainedCurveId();
			AcConstraintGroupNode* pNode = pGroup->getGroupNodePtr(nodeId);
			AcConstrainedCurve* pCurve = AcConstrainedCurve::cast(pNode);

			AcDbObjectId id = pCurve->geomDependencyId();
			

			if(id != AcDbObjectId::kNull)
			{
				AcDbObjectPointer<AcDbAssocGeomDependency> pDependency(id, kForRead);

				HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pDependency), id, parent, false);
				dependencyObjectId = pDependency->dependentOnObject();

				AcDbObjectPointer<AcDbObject> pObject(dependencyObjectId, kForRead);
				addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), dependencyObjectId, item, false);
			}
		}
	}
	else if(pGeometry->isKindOf(AcConstrainedCurve::desc()) == true)
	{
		AcDbObjectId id = pGeometry->geomDependencyId();

		if(id != AcDbObjectId::kNull)
		{

			AcDbObjectPointer<AcDbAssocGeomDependency> pDependency(id, kForRead);
			HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pDependency, str), id, parent, false);

			dependencyObjectId = pDependency->dependentOnObject();	
			AcDbObjectPointer<AcDbObject> pObject(dependencyObjectId, kForRead);
			addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), dependencyObjectId, item, false);

		}
	}
	else if(pGeometry->isKindOf(AcConstrainedRigidSet::desc()) == true)
	{
		ASSERT(FALSE);
		//not sure....
		AcDbObjectId id = pGeometry->geomDependencyId();

		if(id != AcDbObjectId::kNull)
		{
			CString str = _T("");
			AcDbObjectPointer<AcDbAssocGeomDependency> pDependency(id, kForRead);
			HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pDependency, str), id, parent, false);

			dependencyObjectId = pDependency->dependentOnObject();	
			AcDbObjectPointer<AcDbObject> pObject(dependencyObjectId, kForRead);
			addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), dependencyObjectId, item, false);

		}

	}

}

void CConstrainListDlg::addDimDependencyDependencyObjects(AcConstrainedGeometry *pGeometry, HTREEITEM parent)
{
	AcArray<AcGeomConstraint*> apConstraints;
	pGeometry->getConnectedConstraints(apConstraints);

	for(int nIndex = 0; nIndex < apConstraints.length(); nIndex++)
	{
		AcGeomConstraint *pGeomConstraint = apConstraints.at(nIndex);

		if(pGeomConstraint->isActive() == false)
		{
			continue;
		}

		if(pGeomConstraint->isEnabled() == false)
		{
			continue;
		}

		if(pGeomConstraint->isInternal() == true)
		{
			continue;
		}

		if(pGeomConstraint->isKindOf(AcExplicitConstraint::desc()) == true)
		{
			 //AcAngleConstraint
             //AcDistanceConstraint
             //AcRadiusDiameterConstraint

			//dimension constrain
			AcExplicitConstraint *pDimConstrain = (AcExplicitConstraint*)pGeomConstraint;
			AcDbObjectId id = pDimConstrain->dimDependencyId();
			
			
			if(id != AcDbObjectId::kNull)
			{
				CString str = _T("");
				AcDbObjectPointer<AcDbAssocDependency> pDependency(id, kForRead);
				HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pDependency, str), id, parent, false);


				AcDbObjectId  objectId = pDependency->dependentOnObject();
				AcDbObjectPointer<AcDbObject> pObject(objectId, kForRead);
				addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), objectId, item, false);

			}

			id = pDimConstrain->valueDependencyId();

			if(id != AcDbObjectId::kNull)
			{
				CString str = _T("");
				AcDbObjectPointer<AcDbAssocValueDependency> pDependency(id, kForRead);
				HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pDependency, str), id, parent, false);

				AcDbObjectId  objectId = pDependency->dependentOnObject();
				AcDbObjectPointer<AcDbObject> pObject(objectId, kForRead);
				addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), objectId, item, false);

			}
		}
	}

}


void CConstrainListDlg::AddGeomConstraint(AcDbAssoc2dConstraintGroup *pGroup, HTREEITEM parent)
{
	AcArray<AcGeomConstraint*> apConstraints;
	pGroup->getConstraints(apConstraints) ;

	int nLength = apConstraints.length();
	int nIndex = 0;

	for(int nIndex = 0; nIndex < apConstraints.length(); nIndex++)
	{
		AcGeomConstraint *pGeomConstraint = apConstraints.at(nIndex);

		
		if(pGeomConstraint->isActive() == false)
		{
			continue;
		}

		if(pGeomConstraint->isEnabled() == false)
		{
			continue;
		}

		if(pGeomConstraint->isInternal() == true)
		{
			continue;
		}

		AcArray<AcConstrainedGeometry*> apConsGeoms;
		pGeomConstraint->getConnectedGeometries(apConsGeoms);
		
		for(int nGeoIndex = 0; nGeoIndex < apConsGeoms.length(); nGeoIndex++)
		{
			AcConstrainedGeometry *pGeometry = (AcConstrainedGeometry *)apConsGeoms.at(nGeoIndex);
			
			HTREEITEM constrainedGeometryItem = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pGeometry), pGroup->objectId(), parent, false, true, pGeometry->nodeId());
			//add the DependencyId
			addGeometryDependencyObjects(pGeometry, pGroup, constrainedGeometryItem);
			//add DimDependency & value Dependency
			addDimDependencyDependencyObjects(pGeometry, constrainedGeometryItem);

		}
	}
}

void CConstrainListDlg::OnCbnSelchangeComboSpace()
{
	// TODO: Add your control notification handler code here
	AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
	
	CString str = _T("");
	int nIndex = m_SpaceComboBox.GetCurSel();
	m_SpaceComboBox.GetLBText(nIndex, str);

	//load all the block table 
	AcDbObjectPointer<AcDbBlockTable> blockTbl(pDatabase->blockTableId(), kForRead);

	AcDbBlockTableIterator* pIterator = NULL;
	blockTbl->newIterator(pIterator);
	AcDbObjectId Id;
	const TCHAR* symName;
	while(pIterator->done() == false)
	{
		pIterator->getRecordId(Id);

		AcDbObjectPointer<AcDbBlockTableRecord> bTableRecord(Id, kForRead);
	
		bTableRecord->getName(symName);

		//add the name
		if(_tcsicmp(str, symName) == 0)
		{
			m_currentSpaceId = Id;
			break;
		}
		
		
		pIterator->step();
	}

	delete pIterator;


	InitializeDisplay();
}


void CConstrainListDlg::InitializeDisplay()
{
	//clean the list
	for(int nIndex = 0; nIndex < m_treeObjectArray.length(); nIndex++)
	{
		CConstrainTreeItem *pItem = m_treeObjectArray.at(nIndex);
		delete pItem;
	}

	m_treeObjectArray.removeAll();

	//clean the tree control
	m_ConstrainTree.DeleteAllItems();

	//clean the list control
	m_ConstrainList.DeleteAllItems();

	AcDbDatabase *pDatabase = acdbHostApplicationServices()->workingDatabase();
	AcDbObjectId networkId = getAssocNetwork(false, m_currentSpaceId, pDatabase);

	//to list entities contraint command
	if (m_obj != NULL)
	{
		
		AcDbObjectIdArray actionIds;
		AcDbAssocAction::getActionsDependentOnObject(m_obj, true, true, actionIds);

		CString str = _T("");						
		HTREEITEM ConstraintGroupItem = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(m_obj, str), m_obj->objectId(), TVI_ROOT, false);

		for(int i = 0; i < actionIds.length(); ++i)
		{

			AcDbObjectPointer<AcDbAssocAction> pAction(actionIds[i], AcDb::kForRead);

			if (pAction.openStatus() != eOk)
			 continue;
			
			//CString str;
			//CConstrainDbgHelper::objToClassAndHandleStr(pAction, str);
			//acutPrintf(_T("\n Class = %s\n "), str);		

			if(pAction->isKindOf(AcDbAssoc2dConstraintGroup::desc()) == true)
			{
				
				AcDbAssoc2dConstraintGroup*  pConsGrp = 	(AcDbAssoc2dConstraintGroup*)(pAction.object());
				
				if (!pConsGrp)
					 continue;				
							
				AcArray<AcGeomConstraint*> apConstraints;
				pConsGrp->getConstraints(apConstraints) ;

				int nLength = apConstraints.length();
				int nIndex = 0;

				
				for(int nIndex = 0; nIndex < apConstraints.length(); nIndex++)
				{
					AcGeomConstraint *pGeomConstraint = apConstraints.at(nIndex);
												
					if(pGeomConstraint->isActive() == false)
					{
						continue;
					}

					if(pGeomConstraint->isEnabled() == false)
					{
						continue;
					}

					if(pGeomConstraint->isInternal() == true)
					{
						continue;
					}

					HTREEITEM GeomConstraintGroup = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pGeomConstraint), pConsGrp->objectId(), ConstraintGroupItem, false, true, pGeomConstraint->nodeId());

					AcArray<AcConstrainedGeometry*> apConsGeoms;
					pGeomConstraint->getConnectedGeometries(apConsGeoms);
					
					for(int nGeoIndex = 0; nGeoIndex < apConsGeoms.length(); nGeoIndex++)
					{						

						AcConstrainedGeometry *pGeometry = (AcConstrainedGeometry *)apConsGeoms.at(nGeoIndex);

						HTREEITEM constrainedGeometryItem = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pGeometry), pConsGrp->objectId(), GeomConstraintGroup, false, true, pGeometry->nodeId());


						//
						AcDbObjectPointer<AcDbObject> ConstraintEnt(m_obj->objectId(), AcDb::kForRead);
						AcDbObjectIdArray dependencyIds;
						AcDbAssocDependency::getDependenciesOnObject(ConstraintEnt, true, true, dependencyIds);
						AcDbObjectId dependentObjectId;
						AcDbAssocGeomDependency*  pDependency;
						
						//let check the first item - it should have only one dependent object
						AcDbObjectPointer<AcDbObject> pDepObject(dependencyIds[0], AcDb::kForRead);
						if (pDepObject.openStatus() != eOk)
						continue;

						if(pDepObject->isKindOf(AcDbAssocGeomDependency::desc()) == true)
						{								
								 pDependency = 	(AcDbAssocGeomDependency*)(pDepObject.object());
								dependentObjectId = pDependency->dependentOnObject();

								HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pDependency), dependentObjectId, constrainedGeometryItem, false);
								return;
						}
						
						if(pDepObject->isKindOf(AcDbAssocValueDependency::desc()) == true)
						{
								// <<PlaceHolder for later compilition	- Varadan >>					
						}											
												
						//for(int nGeoIndex = 0; nGeoIndex < apConsGeoms.length(); nGeoIndex++)
						//{
						//	AcConstrainedGeometry *pGeometry = (AcConstrainedGeometry *)apConsGeoms.at(nGeoIndex);
							

								//HTREEITEM constrainedGeometryItem = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pGeometry), pConsGrp->objectId(), ConstraintGroupItem, false, true, pGeometry->nodeId());
								//addGeometryDependencyObjects(pGeometry, pConsGrp, constrainedGeometryItem);

								//AcDbObjectId dependencyObjectId = pDependency->dependentOnObject();

								//AcDbObjectPointer<AcDbAssocGeomDependency> pDependency(dependencyObjectId, kForRead);

//								HTREEITEM item = addOneTreeItem(CConstrainDbgHelper::objToClassStr(pDependency), dependentObjectId, ConstraintGroupItem, false);
							
								//AcDbObjectPointer<AcDbObject> pObject(dependentObjectId, kForRead);
								//addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pObject, str), dependentObjectId, item, false);

							
							
						//}

					}
				}
			}			
		}

	}
	else // list constraints
	{
		if(networkId != AcDbObjectId::kNull)
		{
			HTREEITEM newtWorkItem ;
		
						//add the main node
					{
						CString str = _T("");
						AcDbObjectPointer<AcDbAssocNetwork> pNetwork(networkId, kForRead);
						newtWorkItem = addOneTreeItem(CConstrainDbgHelper::objToClassAndHandleStr(pNetwork.object(), str),  networkId, TVI_ROOT, false);
					}

			//add all the AcDbAssocVariable and AcDbAssoc2dConstraintGroup
			AddAssocAction(networkId, newtWorkItem);

			m_ConstrainTree.SelectItem(newtWorkItem);
			m_ConstrainTree.Expand(newtWorkItem, TVE_EXPAND);
		}
	}


	

}

//list control functiom
void CConstrainListDlg::display(AcDbObject* obj)
{
    ASSERT(obj != NULL);

    CString str, str2;

    addSeperatorLine(_T("AcDbObject"));

    addToDataList(_T("Class Name"), CConstrainDbgHelper::objToClassStr(obj));
    addToDataList(_T("Entity Name"), CConstrainDbgHelper::objToEnameStr(obj, str));
    addToDataList(_T("Handle"), CConstrainDbgHelper::objToHandleStr(obj, str));
    addToDataList(_T("Database"), CConstrainDbgHelper::dbToStr(obj->database(), str));

    AcDbObject* ownerObj;
    Acad::ErrorStatus es = acdbOpenObject(ownerObj, obj->ownerId(), AcDb::kForRead);
    if (es == Acad::eOk)
	{
        CConstrainDbgHelper::objToClassAndHandleStr(ownerObj, str);
        ownerObj->close();
        addToDataList(_T("Owner"), str);
    }
    else
	{
        addToDataList(_T("Owner"), CConstrainDbgHelper::rxErrorStr(es));
	}

    addToDataList(_T("Merge Style"), CConstrainDbgHelper::mergeStyleToStr(obj->mergeStyle(), str));
    addToDataList(_T("Has Save Version Override"), CConstrainDbgHelper::booleanToStr(obj->hasSaveVersionOverride(), str));

    AcDb::AcDbDwgVersion dwgVer;
    AcDb::MaintenanceReleaseVersion maintVer;
    es = obj->getObjectBirthVersion(dwgVer, maintVer);
    if (es == Acad::eOk) 
	{
        addToDataList(_T("Birth DWG Version"), CConstrainDbgHelper::dwgVersionToStr(dwgVer, str));
        addToDataList(_T("Birth Maintenance Version"), CConstrainDbgHelper::intToStr(maintVer, str));
    }
    else
	{
        addToDataList(_T("Birth Version"), CConstrainDbgHelper::rxErrorStr(es));
	} 

	addSeperatorLine(_T(""));

	if(obj->isKindOf(AcDbAssocAction::desc()) == true)
	{
		display_AcDbAssocAction((AcDbAssocAction*)(obj));
	}
	if(obj->isKindOf(AcDbAssocNetwork::desc()) == true)
	{
		display_AcDbAssocNetwork((AcDbAssocNetwork*)(obj));
	}
	if(obj->isKindOf(AcDbAssocVariable::desc()) == true)
	{
		display_AcDbAssocVariable((AcDbAssocVariable*)(obj));
	}
}

void CConstrainListDlg::display_AcDbAssocAction(AcDbAssocAction * obj)
{
	addSeperatorLine(_T("AcDbAssocAction"));
	addSeperatorLine(_T(""));
}

void CConstrainListDlg::display_AcDbAssocNetwork(AcDbAssocNetwork * obj)
{
	addSeperatorLine(_T("AcDbAssocNetwork"));
	addSeperatorLine(_T(""));

}


void CConstrainListDlg::display_AcDbAssocVariable(AcDbAssocVariable *obj)
{
	addSeperatorLine(_T("AcDbAssocVariable"));

	CString string = _T("");
	AcString str = obj->name();
	addToDataList(_T("Name") , str.constPtr());

	str = obj->expression();
	addToDataList(_T("Expression") , str.constPtr());

	AcDbEvalVariant val = obj->value ();
	AcDb::DwgDataType type = val.getType();


	CConstrainDbgHelper::dwgDataTypeToStr(type, string);
	addToDataList(_T("Data type") , string);

	CConstrainDbgHelper::AcDbEvalVariantToStr(val, string);
	addToDataList(_T("Data value") , string);

	AcDbObjectIdArray dependencyIds;
	obj->getActionsDependentOnObject(obj, true, true, dependencyIds);

	bool UserVariable = true;

	for (int nIndex1 = 0; nIndex1 < dependencyIds.length(); nIndex1++)
	{
		AcDbObjectId id = dependencyIds.at(nIndex1);

		AcDbObjectPointer<AcDbObject> pAction1(id, kForRead);
		if (pAction1.openStatus() != eOk)
			break;

		if(pAction1->isKindOf(AcDbAssoc2dConstraintGroup::desc()) == true)
		{
			UserVariable = false;
		}
	}

	
	addToDataList(_T("User variable"), CConstrainDbgHelper::booleanToStr(UserVariable, string));
	
	str = obj->description();
	addToDataList(_T("Description") , str.constPtr());

	addSeperatorLine(_T(""));
}



void CConstrainListDlg::display_AcConstraintGroupNode(AcConstraintGroupNode *pObj)
{
	addSeperatorLine(_T("AcConstraintGroupNode"));
	
	//name
	CString string = _T("");
	AcString str = pObj->isA()->name();
	addToDataList(_T("Name") , str.constPtr());

	//nodeId
	AcConstraintGroupNodeId  nodeId = pObj->nodeId();
	addToDataList(_T("Constraint Group NodeId"), CConstrainDbgHelper::intToStr(nodeId, string));

	//owner
	AcDbObjectId owner = pObj->owningConstraintGroupId();
	{
		AcDbObjectPointer<AcDbObject> ownerObj(owner, kForRead);
		CConstrainDbgHelper::objToClassAndHandleStr(ownerObj, string);
		addToDataList(_T("Owner constraint group"), string);
	}

	addSeperatorLine(_T(""));

	if(pObj->isKindOf(AcConstrainedGeometry::desc()) == true)
	{
		display_AcConstrainedGeometry((AcConstrainedGeometry*)pObj);
	}
	else if(pObj->isKindOf(AcGeomConstraint::desc()) == true)
	{
	}
	else if(pObj->isKindOf(AcHelpParameter::desc()) == true)
	{
	}
	else
	{
		ASSERT(FALSE);
	}

}

void CConstrainListDlg::display_AcConstrainedGeometry(AcConstrainedGeometry *pObj)
{
	addSeperatorLine(_T("AcConstrainedGeometry"));

	 CString str = _T("");
	//is read only (fixed?)
	addToDataList(_T("Is read only"), CConstrainDbgHelper::booleanToStr(pObj->isReadOnly(), str));

	//is Owning RigidSet 
	AcConstrainedRigidSet* pRigitSet = pObj->getOwningRigidSet() ;

	if(pRigitSet != NULL)
	{
		addToDataList(_T("Is Owning rigid set"), CConstrainDbgHelper::booleanToStr(false, str));
	}
	else
	{
		addToDataList(_T("Is Owning rigid set"), CConstrainDbgHelper::booleanToStr(true, str));
	}


	addSeperatorLine(_T(""));

    //AcConstrainedCurve
    //AcConstrainedPoint
    //AcConstrainedRigidSet
	if(pObj->isKindOf(AcConstrainedCurve::desc()) == true)
	{
		display_AcConstrainedCurve((AcConstrainedCurve*)pObj);
	}
	else if(pObj->isKindOf(AcConstrainedPoint::desc()) == true)
	{
	}
	else if(pObj->isKindOf(AcConstrainedRigidSet::desc()) == true)
	{
	}
	else
	{
		ASSERT(FALSE);
	}
}


void CConstrainListDlg::display_AcConstrainedCurve(AcConstrainedCurve *pObj)
{
	addSeperatorLine(_T("AcConstrainedCurve"));

	CString string = _T("");
	//get getConstrainedImplicitPoints
	AcArray<AcConstrainedImplicitPoint*> apImplicitPoints;
	pObj->getConstrainedImplicitPoints(apImplicitPoints);

	for(int nIndex = 0;nIndex < apImplicitPoints.length();nIndex++)
	{
		addSeperatorLine(_T("Implicit Points "));
		
		AcConstrainedImplicitPoint *pPoints = apImplicitPoints.at(nIndex);

		const ACHAR* szNme = pPoints->isA()->name();
		addToDataList(_T("Name") , szNme);

		AcConstraintGroupNodeId  nodeId = pPoints->constrainedCurveId();
		addToDataList(_T("Constraint Group NodeId"), CConstrainDbgHelper::intToStr(nodeId, string));

		//point
		AcGePoint3d pt = pPoints->point();
		addToDataList(_T("Point"), CConstrainDbgHelper::ptToStr(pt, string));

		//point index
		int nPtIndex = pPoints->pointIndex();
		addToDataList(_T("Point index"), CConstrainDbgHelper::intToStr(nPtIndex, string));

		//point type
		AcConstrainedImplicitPoint::ImplicitPointType type = pPoints->pointType();
		addToDataList(_T("Point type"), CConstrainDbgHelper::ImplicitPointTypeToStr(type, string));
	}

	//isBounded()
	addToDataList(_T("is Bounded"), CConstrainDbgHelper::booleanToStr(pObj->isBounded(), string));

	addSeperatorLine(_T(""));

	//AcConstrainedCircle
	//AcConstrainedEllipse
	//AcConstrainedLine
	//AcConstrainedSpline
	if(pObj->isKindOf(AcConstrainedCircle::desc()) == true)
	{
		display_AcConstrainedCircle((AcConstrainedCircle*)pObj);
	}
	else if(pObj->isKindOf(AcConstrainedEllipse::desc()) == true)
	{
	}
	else if(pObj->isKindOf(AcConstrainedLine::desc()) == true)
	{
	}
	else if(pObj->isKindOf(AcConstrainedSpline::desc()) == true)
	{
	}
	else
	{
		ASSERT(FALSE);
	}
}

void CConstrainListDlg::display_AcConstrainedCircle(AcConstrainedCircle *pObj)
{
	addSeperatorLine(_T("AcConstrainedCircle"));

	CString string = _T("");

	//center point
	AcGePoint3d pt = pObj->centerPoint();
	addToDataList(_T("Center point"), CConstrainDbgHelper::ptToStr(pt, string));

	double dradius = pObj->radius();
	addToDataList(_T("Radius"), CConstrainDbgHelper::doubleToStr(dradius, string));


	addSeperatorLine(_T(""));
	//AcConstrainedArc 
	if(pObj->isKindOf(AcConstrainedArc::desc()) == true)
	{
		display_AcConstrainedArc((AcConstrainedArc*)pObj);
	}

}


void CConstrainListDlg::display_AcConstrainedArc(AcConstrainedArc* pObj)
{
	CString string = _T("");
	addSeperatorLine(_T("AcConstrainedArc"));

	AcGePoint3d pt = pObj->startPoint();
	addToDataList(_T("Start point"), CConstrainDbgHelper::ptToStr(pt, string));

	pt = pObj->endPoint();
	addToDataList(_T("End point"), CConstrainDbgHelper::ptToStr(pt, string));

	pt = pObj->midPoint();
	addToDataList(_T("Mid point"), CConstrainDbgHelper::ptToStr(pt, string));

	addSeperatorLine(_T(""));

}

void CConstrainListDlg::buildColumns(CListCtrl& dataList)
{
        // get the width of the window
    CRect lvRect;
    int col1_width, col2_width;
    dataList.GetClientRect(lvRect);
    col1_width = lvRect.Width() / 2;
    col2_width = lvRect.Width() - col1_width - ::GetSystemMetrics(SM_CXVSCROLL);

        // initialize the columns
    LV_COLUMN lvc;
    lvc.mask = LVCF_WIDTH | LVCF_SUBITEM | LVCF_TEXT;
    lvc.cx = col1_width;
    lvc.pszText = _T("Field");
    lvc.iSubItem = 0;
    dataList.InsertColumn(0, &lvc);

    lvc.cx = col2_width;
    lvc.iSubItem = 1;
    lvc.pszText = _T("Value");
    dataList.InsertColumn(1, &lvc);
}


/****************************************************************************
**
**  CConstrainListDlg::addToDataList
**
**  **jma
**
*************************************/

void CConstrainListDlg::addToDataList(LPCTSTR field, LPCTSTR value, bool addQuotes)
{
    if (field == NULL)
		m_fieldStrList.AddTail(_T(""));
	else
		m_fieldStrList.AddTail(field);

    if (value == NULL) {
        m_valueStrList.AddTail(_T("(NULL)"));    // some Acad funcs return Null strings!
    }
    else {
        if (addQuotes) {
            CString str;
            str.Format(_T("\"%s\""), value);
            m_valueStrList.AddTail(str);
        }
        else
            m_valueStrList.AddTail(value);
    }
}

/****************************************************************************
**
**  CConstrainListDlg::addSeperatorLine
**
**  **jma
**
*************************************/

void CConstrainListDlg::addSeperatorLine(LPCTSTR className)
{
    CString str = _T("");
	CString strName(className);

	if(strName.IsEmpty() == false)
		str.Format(_T("------ %s ------"), className);

    addToDataList(str, _T(""));
}

/****************************************************************************
**
**  ArxDbgUiTdcRxObjectBase::drawPropsList
**
**  **jma
**
*************************************/

void CConstrainListDlg::drawPropsList(CListCtrl& dataList)
{
	this->LockWindowUpdate();		// suppress redraw events

    dataList.DeleteAllItems();		// remove any existing items from the control

        // build up the list view control
    LV_ITEM lvi;
    lvi.mask = 0;
    int row = 0;

    POSITION pos1 = m_fieldStrList.GetHeadPosition();
    POSITION pos2 = m_valueStrList.GetHeadPosition();

    while ((pos1 != NULL) && (pos2 != NULL)) {
        lvi.iItem = row;
        lvi.iSubItem = 0;
        dataList.InsertItem(&lvi);
        dataList.SetItemText(row, 0, m_fieldStrList.GetNext(pos1));
        dataList.SetItemText(row, 1, m_valueStrList.GetNext(pos2));
        row++;
    }

	this->UnlockWindowUpdate();		// flushes redraw events
}
//list control function

